# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from .exceptions import InvalidQueryError


class QueryBuilder(object):
    def __init__(self, entity):
        self.entity = entity
        self.columns = '*'
        self.filterflag = False
        self.filters = []
        self.incomplete_filter_flag = False
        self.countflag = False

        self.paginationflag = False
        self.maxresults = 100
        self.startposition = 1

    def get_entity(self):
        return self.entity

    def select(self, columns):
        if isinstance(columns, (tuple, list)):
            self.columns = ", ".join(columns)
        else:
            self.columns = columns
        return self

    def get_columns(self):
        return self.columns if self.columns else '*'

    def where(self, column):
        self._validate()
        self.incomplete_filter_flag = True
        self.filters.append(column)
        return self

    def equals(self, value):
        if value is None:
            value = ''

        column = self.filters[-1]
        if isinstance(value, bool):
            self.filters[-1] = "%s = %s" % (column, str(value).lower())

        else:
            self.filters[-1] = "%s = '%s'" % (column, value)

        self.incomplete_filter_flag = False
        return self

    def contains(self, values):
        if not isinstance(values, (list, tuple)):
            raise InvalidQueryError("Contains operator must "
                                    "receive a list/tuple of values")

        column = self.filters[-1]
        self.filters[-1] = "%s in %s" % (column, values)
        self.incomplete_filter_flag = False
        return self

    def gt(self, value):
        return self._operator('>', value)

    def gte(self, value):
        return self._operator('>=', value)

    def lt(self, value):
        return self._operator('<', value)

    def lte(self, value):
        return self._operator('<=', value)

    def _operator(self, op, value):
        try:
            float(value)
        except ValueError:
            raise InvalidQueryError("%s operator requires "
                                    "value of type integer/float" % op)

        column = self.filters[-1]
        self.filters[-1] = "%s %s '%s'" % (column, op, value)
        self.incomplete_filter_flag = False
        return self

    def like(self, value):
        if not isinstance(value, basestring):
            raise InvalidQueryError("Like operator requires "
                                    "value be of type string")

        column = self.filters[-1]
        self.filters[-1] = "%s Like '%s'" % (column, value)
        self.incomplete_filter_flag = False
        return self

    def set_filters(self, where):
        self.filters = where.split(' AND ')
        self._validate()

    def get_filters(self):
        return self.filters

    def count(self):
        self._validate()
        self.countflag = True
        self.columns = "count(*)"
        return self

    def is_count_query(self):
        return self.countflag

    def limit(self, maxresults):
        self._validate()
        self.paginationflag = True
        self.maxresults = maxresults
        return self

    def get_maxresults(self):
        return self.maxresults

    def offset(self, startposition):
        self._validate()
        self.paginationflag = True
        self.startposition = startposition
        return self

    def get_startposition(self):
        return self.startposition
    
    
    @classmethod
    def from_parts(cls, entity, columns='*', where=None, count=False, 
                   maxresults=100, startposition=0):
        
        qb = QueryBuilder(entity)
        qb.select(columns)
        qb.limit(maxresults)
        qb.offset(startposition)
        if count:
            qb.count()

        if where:
            qb.set_filters(where)

        return qb

    def build(self):
        self._validate()
        if not self.columns:
            self.columns = '*'

        query = "Select %s From %s" % (self.columns, self.entity)

        if self.filters:
            query += ' Where %s' % ' AND '.join(self.filters)

        if self.paginationflag:
            query += ' StartPosition %d MaxResults %d' % (self.startposition,
                                                          self.maxresults)

        return query

    def _validate(self):
        if self.incomplete_filter_flag:
            raise InvalidQueryError("Incomplete Filter Clause - > %s" %
                                    self.filters[-1])





# -*- coding: utf-8 -*-

"""
quickbook3.auth
~~~~~~~~~~~~~

This module contains a quickbook authentication service that implements the
OAuth 1.0/a auth flow using rauth.OAuth1Service
"""


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from rauth import OAuth1Service


class QuickBookAuthService(object):
    """
    A quickbook authentication service that is actually a wrapper around the
    :class:`~rauth.service.OAuth1Service` object. Primarily this wrapper is used
    to produce authenticated session objects.

    You can initialize :class:`QuickBookAuthService` as follows::

        service = QuickBookAuthService(consumer_key='123',
                                       consumer_secret='456',
                                       callback_url=http://example.com/authorize')

    where the callback_url is the redirect url where user is redirected by
    quicksbook once the user has authorized the application.

    Now the request token should be retrieved::

        request_token, request_token_secret = service.get_request_token_pair()

    This request token pair must be now saved in the user session. You then
    access the authorize URI and direct the client to authorize the requests
    on their behalf. This URI is retrieved as follows::

        authorize_url = service.get_authorize_url()

    Once the client has authorized the request, quickbook server redirects the
    user to callback_url specified during initialization. From the
    querystring of the callback url, we get the realmID(companyid), request token
    and the oauth_verifier. This request token can be verified against the one
    stored in the user session for securtiy purposes.

    You can now create a new instance of auth service by passing the consumer
    key pair as well as the request token pair obtained above::

        service = QuickBookAuthService(consumer_key='123',
                                       consumer_secret='456',
                                       request_token='789'
                                       request_token_secret='abc'
                                       callback_url=http://example.com/authorize')


    And obtain the access token pair as follows::

        access_token, access_token_secret = service.get_access_tokens()

    Using these, now you can access an authenticated session object as follows::

        session = OAuth1Session(consumer_key='123',
                                consumer_secret='456',
                                access_token='678',
                                access_token_secret='abc')


    Finally you have an authenticated session and are ready to make requests
    against Quickbooks api endpoints.

    """

    request_token_url = "https://oauth.intuit.com/oauth/v1/get_request_token"
    access_token_url = "https://oauth.intuit.com/oauth/v1/get_access_token"

    authorize_url = "https://appcenter.intuit.com/Connect/Begin"

    def __init__(self, consumer_key, consumer_secret,
                 callback_url, request_token=None, request_token_secret=None):

        """

        :param consumer_key: Client consumer key, required for signing.
        :type consumer_key: str
        :param consumer_secret: Client consumer secret, required for signing.
        :type consumer_secret: str
        :param callback_url: Redirect url where quickbook must redirect after
            authentication
        :type callback_url: str
        :param request_token: Request Token as returned by
            :class:`get_request_token_pair`.
        :type request_token: str
        :param request_token_secret: Request Token Secret as returned by
            :class:`get_request_token_pair`.
        :type request_token_secret: str

        """

        self.qbservice = OAuth1Service(name=None,
                                       consumer_key=consumer_key,
                                       consumer_secret=consumer_secret,
                                       request_token_url=self.request_token_url,
                                       access_token_url=self.access_token_url,
                                       authorize_url=self.authorize_url,
                                       base_url=None)

        self.callback_url = callback_url
        self.request_token = request_token
        self.request_token_secret = request_token_secret

    def get_request_token_pair(self):
        """
        Return a request token pair.
        """

        self.request_token, self.request_token_secret = \
            self.qbservice.get_request_token(
                params={'oauth_callback': self.callback_url})

        return self.request_token, self.request_token_secret

    def get_authorize_url(self):
        """
        Returns a formatted authorize URL.
        """

        if not self.request_token:
            self.get_request_token_pair()

        return self.qbservice.get_authorize_url(self.request_token)

    def get_access_tokens(self, oauth_verifier):
        """
        Returns an access token pair.

        :param oauth_verifier: A string returned by quickbooks server as
        querystring parameter and used to verify that request for access token is
        indeed coming from the application server.
        :type string:
        """

        return self.qbservice.get_access_token(
            self.request_token, self.request_token_secret,
            data={'oauth_verifier': oauth_verifier})




# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


class QuickBooksError(Exception):
    """
    A base exception for quickbooks-related errors
    """
    pass


class MissingCredentialsException(QuickBooksError):
    """
    Raised when either of credentials ie (consumer key, consumer secret,
    access token or access token secret) is missing
    """

    def __init__(self):
        super(MissingCredentialsException, self)\
            .__init__("All of consumer key, consumer secret "
                      "access token and access secret must be passed")


class InvalidQueryError(QuickBooksError):
    """
    Raised while constructing an invalid query using the querybuilder
    """
    pass


class HttpQuickBookError(QuickBooksError):
    """
    A base exception for http=related errors returned from quickbooks
    """
    pass


class AuthenticationError(HttpQuickBookError):
    """
    Raised when authentication to quickbook server fails.
    Corresponding to http status code 401
    """
    def __init__(self, *args):
        super(AuthenticationError, self).__init__('User authentication Failed')


class PermissionDenied(HttpQuickBookError):
    """
    Raised when the user doesn't have permission to access this resource.
    Corresponding to http status code 403
    """
    def __init__(self, *args):
        super(PermissionDenied, self).\
            __init__("User doesn't have permission to access this resource")


class NotFoundError(HttpQuickBookError):
    """
    Raised when the requested resource is not available.
    Corresponding to http status code 400
    """

    def __init__(self, *args):
        super(NotFoundError, self).__init__('Resource Not Found')


class ServerError(HttpQuickBookError):
    """
    Raised when there is some unknow error from the server.
    Corresponding to http status code 500
    """
    pass


class ServiceUnavailable(HttpQuickBookError):
    """
    Raised when server is down or not available.
    Corresponding to http status code 503
    """
    def __init__(self, *args):
        super(ServiceUnavailable, self).__init__('Service Unavailable')


class UnknownError(HttpQuickBookError):
    """
    Raised corresponding to all other http error codes
    """

    def __init__(self, status_code, error):
        super(UnknownError, self).__init__(
            "Status Code: %d. Reason: %s" % (status_code, error))


class GenericError(HttpQuickBookError):
    """
    A base exception for quickbook specific error which is denoted by Fault.
    """

    def __init__(self, errors):
        self.errors = errors
        error_message = 'Validation Error:'
        for error in self.errors:
            error_message += ' %s.' % error['Detail']

        super(HttpQuickBookError, self).__init__(error_message)


class ValidationFault(GenericError):
    """
    Raised when some invalid data have been sent.
    """
    pass


class ServiceError(GenericError):
    """
    A non-recoverable error. Something failed on the server
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_quickbooks-py
----------------------------------

Tests for `quickbooks-py` module.
"""

import unittest

from quickbooks-py import quickbooks-py


class TestQuickbooks-py(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division

import requests
import xmltodict

from .exceptions import AuthenticationError, PermissionDenied, NotFoundError, \
    ServerError, ServiceUnavailable, ValidationFault, UnknownError


class ResponseParser(object):

    HTTP_CODE_EXCEPTION_MAP = {
        401: AuthenticationError,
        403: PermissionDenied,
        404: NotFoundError,
        500: ServerError,
        503: ServiceUnavailable
    }

    FAULT_TYPE_EXCEPTION_MAP = {
        'AUTHENTICATION': AuthenticationError,
        'AUTHORIZATION': PermissionDenied,
        'VALIDATIONFAULT': ValidationFault,
        'SERVICEFAULT': ServerError,
        'SYSTEMFAULT': ServerError
    }

    def __init__(self, response):
        super(ResponseParser, self).__init__()
        self.response = response

    def parse(self):
        status_code = self.response.status_code
        if status_code != requests.codes.ok:
            self.parse_http_error()

        elif self.is_xml_response():
            self.parse_quickbooks_error()

        else:
            json_response = self.response.json()
            if 'Fault' in json_response:
                self.parse_quickbooks_error()
            else:
                return json_response

    def parse_http_error(self):
        status_code = self.response.status_code
        if status_code in self.HTTP_CODE_EXCEPTION_MAP:
            exception = self.HTTP_CODE_EXCEPTION_MAP[status_code]

            raise exception(self.response.reason)

        elif status_code == requests.codes.bad_request:
            return self.parse_quickbooks_error()
        else:
            raise UnknownError(status_code, self.response.reason)

    def parse_quickbooks_error(self):
        if self.is_xml_response():
            parsed_error = xmltodict.parse(self.response.json)
            fault = parsed_error['IntuitResponse']['Fault']
            fault_type = fault['@type'].upper()
            for error in fault['Errors']:
                error['code'] = error['@code']
                del error['@code']

        else:
            fault = self.response.json()['Fault']
            fault_type = fault['type'].upper()

        raise self.FAULT_TYPE_EXCEPTION_MAP[fault_type](fault['Error'])

    def is_xml_response(self):
        return 'xml' in self.response.headers['content-type']


class QueryResponse(object):

    def __init__(self, entity, query_response):
        self.entity = entity
        self.object_list = query_response.get(entity, [])
        self.startposition = query_response.get('startPosition', 1)
        self.maxresults = query_response['maxResults']
        self.total_count = query_response.get('totalCount', self.maxresults)

    def __repr__(self):
        return "Entity: %s, StartPosition: %d, Count: %d, " \
               "MaxResults: %d" % (self.entity, self.startposition,
                                   self.total_count, self.maxresults)


class CDCResponse(object):
    def __init__(self, entities, cdc_response):
        if isinstance(cdc_response, (list, tuple)):
            cdc_response = cdc_response[0]

        self.upsert = {}
        self.delete = {}

        for ind, entity in enumerate(entities):
            self.delete[entity] = []
            self.upsert[entity] = []

            query_response = cdc_response['QueryResponse'][ind]
            for obj in query_response.get(entity, []):
                if not 'status' in obj:
                    self.upsert[entity].append(obj)
                else:
                    self.delete[entity].append(obj)
                    
    def __repr__(self):
        upsert = "Upsert: %s"  % \
                 (str({entity: len(self.upsert[entity])
                       for entity in self.upsert.keys()}))
        
        delete = "Delete: %s"  % \
                 (str({entity: len(self.delete[entity])
                       for entity in self.delete.keys()}))
        
        return "%s, %s" % (upsert, delete)


# -*- coding: utf-8 -*-

"""
    A quickbooks client to access various quickbooks endpointz.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os
import sys
import datetime

from rauth import OAuth1Session

from .response import ResponseParser, QueryResponse, CDCResponse


try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from .exceptions import *
from .querybuilder import *


class QuickBooks(object):
    production_url = "https://quickbooks.api.intuit.com/v3"

    sandbox_url = "https://sandbox-quickbooks.api.intuit.com/v3"

    ACCOUNTING_SERVICES = [
        "Account", "Attachable", "Bill", "BillPayment",
        "Class", "CompanyInfo", "CreditMemo", "Customer",
        "Department",
        "Deposit",
        "Employee", "Estimate", "Invoice",
        "Item", "JournalEntry", "Payment", "PaymentMethod",
        "Preferences", "Purchase", "PurchaseOrder",
        "SalesReceipt", "TaxCode", "TaxRate", "Term",
        "TimeActivity", "Vendor", "VendorCredit"
    ]

    CONSUMER_KEY_NAME = 'QB_CONSUMER_KEY'
    CONSUMER_SECRET_NAME = 'QB_CONSUMER_SECRET'
    ACCESS_TOKEN_NAME = 'QB_ACCESS_TOKEN'
    ACCESS_TOKEN_SECRET_NAME = 'QB_ACCESS_TOKEN_SECRET'

    def __init__(self, company_id, consumer_key=None, consumer_secret=None,
                 access_token=None, access_token_secret=None,
                 cred_file=None, sandbox_mode=False,
                 logger=None, peform_logging=False,
                 log_level=logging.ERROR):

        """
        :param company_id: This is the realmID obtained during authorization
        :type company_id: str
        :param consumer_key: Client consumer key, required for signing,
                             defaults to `None`.
        :type consumer_key: str
        :param consumer_secret: Client consumer secret, required for signing.
                                defaults to `None`.
        :type consumer_secret: str
        :param access_token: Access token, defaults to `None`.
        :type access_token: str
        :param access_token_secret: Access token secret, defaults to `None`.
        :type access_token_secret: str
        :param cred_file: Path to the credential file to read the consumer and
                          access token pair, defaults to `None`.

        :param sandbox_mode: A flag indicating whether to use production
            endpoints or sandbox endpoints, , defaults to `False`.
        :type sandbox_mode: bool
        :param logger: A logger to be used to log messages, error and debugging
            information
        :type logger: :class:`logging`

        :param peform_logging: A flag indicating whether to perform Logging,
            defaults to `True`.
        :type peform_logging: bool

        :param log_level: Mininum Log Level to log
        :type log_level: int
        :return:
        """

        self.sandbox_mode = sandbox_mode

        self.base_url_v3 = self.sandbox_url if self.sandbox_mode \
            else self.production_url

        self.company_id = company_id

        if cred_file:
            self._read_creds_from_file(cred_file)

        else:
            self._get_credentials(consumer_key, consumer_secret,
                                  access_token, access_token_secret)

        self.peform_logging = peform_logging

        self.logger = logger

        self.log_level = log_level

        if self.peform_logging and not self.logger:
            self.logger = logging.getLogger()
            self.logger.addHandler(logging.StreamHandler(sys.stdout))
            self.logger.setLevel(self.log_level)

        self._create_session()

    def create(self, resource, resource_dict, **params):
        url = self._get_crud_url(resource)
        response = self._execute(method='post', url=url,
                                 params=params or {},
                                 json=resource_dict)

        return response[resource]

    def read(self, resource, resource_id, **params):
        url = self._get_crud_url(resource, resource_id)
        response = self._execute(method='get', url=url, params=params or {})
        return response[resource]

    def update(self, resource, resource_dict, **params):
        url = self._get_crud_url(resource)
        response = self._execute(method='post', url=url,
                                 params=params or {},
                                 json=resource_dict)
        return response[resource]

    def delete(self, resource, resource_dict, **params):
        url = self._get_crud_url(resource)
        params = params or {}
        params['operation'] = 'delete'
        response = self._execute(method='post', url=url,
                                 params=params,
                                 json=resource_dict)

        return response[resource]

    def query(self, querybuilder, **params):
        query = querybuilder.build()
        entity = querybuilder.get_entity()
        count = querybuilder.is_count_query()
        return self._query(entity, query, count, params)

    def _query(self, entity, query, count=False, params=None):
        params = params or {}

        params['query'] = query

        url = "/".join([self.base_url_v3, 'company', self.company_id, 'query'])

        response = self._execute(method='get', url=url, params=params)

        if count:
            return response['QueryResponse']['totalCount']
        else:
            return QueryResponse(entity, response['QueryResponse'])

    def batch_query(self, querybuilder, **params):
        params = params or {}
        maxresults = querybuilder.get_maxresults()
        while True:
            query_response = self.query(querybuilder, **params)
            total_count = query_response.total_count
            startposition = query_response.startposition
            yield query_response
            if total_count < maxresults:
                return
            else:
                querybuilder.offset(startposition + maxresults)

    def report(self, name, **params):
        params = params or {}

        url = "/".join([self.base_url_v3, 'company', self.company_id,
                        'reports', name])

        response = self._execute(method='get', url=url, params=params)

        return response

    def cdc(self, entities, changed_since):
        if isinstance(changed_since, datetime.datetime):
            changed_since = changed_since.isoformat()

        params = {
            'entities': ','.join(entities),
            'changedSince': changed_since
        }

        url = "/".join([self.base_url_v3, 'company', self.company_id, 'cdc'])

        response = self._execute(method='get', url=url, params=params)

        return CDCResponse(entities, response['CDCResponse'])

    def _execute(self, method, url, **kwargs):
        method = getattr(self.session, method)
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json'}

        response = method(url, header_auth=True,
                          realm=self.company_id, headers=headers,
                          **kwargs)

        return ResponseParser(response).parse()

    def _get_crud_url(self, resource, resource_id=None):
        urlparts = [self.base_url_v3, 'company', self.company_id,
                    resource.lower()]

        if resource_id:
            urlparts.append(resource_id)

        return "/".join(urlparts)

    def _create_session(self):
        self.session = OAuth1Session(self.consumer_key,
                                     self.consumer_secret,
                                     self.access_token,
                                     self.access_token_secret)

    def _get_credentials(self, consumer_key, consumer_secret, access_token,
                         access_token_secret):

        try:
            self.consumer_key = consumer_key or \
                                os.environ[self.CONSUMER_KEY_NAME]
            self.consumer_secret = consumer_secret or \
                                   os.environ[self.CONSUMER_SECRET_NAME]

            self.access_token = access_token or \
                                os.environ[self.ACCESS_TOKEN_NAME]
            self.access_token_secret = access_token_secret or \
                                       os.environ[self.ACCESS_TOKEN_SECRET_NAME]

        except KeyError:
            raise MissingCredentialsException

    def _read_creds_from_file(self, filename):
        config = configparser.ConfigParser()
        config.read(filename)

        try:
            self.consumer_key = config.get('credentials',
                                           self.CONSUMER_KEY_NAME)
            self.consumer_secret = config.get('credentials',
                                              self.CONSUMER_SECRET_NAME)

            self.access_token = config.get('credentials',
                                           self.ACCESS_TOKEN_NAME)
            self.access_token_secret = config.get('credentials',
                                                  self.ACCESS_TOKEN_SECRET_NAME)

        except configparser.NoOptionError:
            raise MissingCredentialsException





#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

with open('requirements.txt') as reqfile:
    requirements = reqfile.readlines()

setup(
    name='quickbooks-py',
    version='0.1.3',
    description="QuickBooks Python Client is a python library for quickbooks api version 3",
    long_description=readme + '\n\n' + history,
    author="Ritesh Kadmawala",
    author_email='ritesh@loanzen.in',
    url='https://github.com/loanzen/quickbooks-py',
    packages=[
        'quickbook3',
    ],
    package_dir={'quicbook3':
                 'quickbook3'},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='quickbooks-py',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
)



# -*- coding: utf-8 -*-

__author__ = 'Ritesh Kadmawala'
__email__ = 'ritesh@loanzen.in'
__version__ = '0.1.3'

from quickbook import *
from querybuilder import *
from auth import *
from exceptions import *
from response import *
